


// Name
//===================================
//
// PlayerMovement
//


// DEPENDENCIES
//===================================

    #include "collision.h"
    #include "input.h"
    #include "player_movement.h"
    #include "sprite.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    PlayerMovement::~PlayerMovement() {};

    void PlayerMovement::update( Sprite& sprite, Input& input )
    {

            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                sprite.duck();
            }
            else
            {
                sprite.unduck();
            }

        if ( input.held( Input::Action::MOVE_LEFT ) && !input.held( Input::Action::MOVE_RIGHT ) && ( !sprite.isDucking() || !sprite.onGround() ) )
        {
            sprite.moveLeft();
        }
        if ( input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ) && ( !sprite.isDucking() || !sprite.onGround() ) )
        {
            sprite.moveRight();
        }
        if ( !input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ))
        {
            if ( sprite.isDucking() )
                sprite.stopDucking();
            else
                sprite.stop();
        }

        if ( input.held( Input::Action::RUN ) )
        {
            sprite.run();
        }

        if ( input.held( Input::Action::EXTRA1 ) )
        {
            sprite.debugResetPos();
        }

        if ( input.held( Input::Action::RUN ) )
        {
            sprite.run();
        }

        if ( input.held( Input::Action::JUMP ) )
        {
                sprite.jump();
                sprite.jump_lock_ = true;

            sprite.slowFall();
        }

        if ( input.released( Input::Action::JUMP ) )
        {
            sprite.jump_lock_ = false;
        }

        if ( collide_top_prev_ && collide_bottom_prev_ )
        {
            if ( sprite.directionX() == Direction::Horizontal::RIGHT )
                sprite.moveRight();
            else
                sprite.moveLeft();
        }

        if ( input.held( Input::Action::MOVE_UP ) || input.held( Input::Action::MOVE_DOWN ) )
        {
            if ( touching_ladder_ )
                sprite.grabLadder();
        }

        if ( input.held( Input::Action::MOVE_UP ) )
        {
            sprite.moveUp();
        }

        if ( input.held( Input::Action::MOVE_DOWN ) )
        {
            sprite.moveDown();
        }

        if ( !touching_ladder_ )
            sprite.releaseLadder();

        collide_top_prev_ = collide_top_;
        collide_bottom_prev_ = collide_bottom_;
        collide_top_ = false;
        collide_bottom_ = false;
        touching_ladder_prev_ = touching_ladder_;
        touching_ladder_ = false;
    };

    void PlayerMovement::collision( Collision collision, Sprite& sprite, Sprite& other )
    {
        /*
        if ( collision.direction() == Direction::Simple::DOWN )
        {
            sprite.collideStopYBottom( collision.overlapY() );
        }
        if ( collision.direction() == Direction::Simple::UP )
        {
            sprite.collideStopYTop( collision.overlapY() );
        }

        if ( collision.direction() == Direction::Simple::LEFT )
        {
            sprite.collideStopXLeft( collision.overlapX() );
        }
        if ( collision.direction() == Direction::Simple::RIGHT )
        {
            sprite.collideStopXRight( collision.overlapX() );
        }


        if ( collision.directionY() == Direction::Vertical::DOWN )
        {
            sprite.collideStopYBottom( collision.overlapY() );
        }
        if ( collision.directionY() == Direction::Vertical::UP )
        {
            sprite.collideStopYTop( collision.overlapY() );
        }

        if ( collision.directionX() == Direction::Horizontal::LEFT )
        {
            sprite.collideStopXLeft( collision.overlapX() );
        }
        if ( collision.directionX() == Direction::Horizontal::RIGHT )
        {
            sprite.collideStopXRight( collision.overlapX() );
        }*/


        if ( other.type() == Sprite::Type::LADDER )
        {

            if ( collision.collideAny() )
            {
                touching_ladder_ = true;
            }
        }

        if ( other.type() == Sprite::Type::FLAT_TOP )
        {

            if ( collision.collideBottom() && sprite.xPrevSubPixels() <= sprite.xSubPixels() + 4000 )
            {
                sprite.collideStopYBottom( collision.overlapYBottom() );
                collide_top_ = true;
            }
        }

        if ( other.type() == Sprite::Type::BLOCK )
        {
            if ( collision.collideLeft() )
            {
                sprite.collideStopXLeft( collision.overlapXLeft() );
            }
            if ( collision.collideRight() )
            {
                sprite.collideStopXRight( collision.overlapXRight() );
            }

            if ( collision.collideBottom() )
            {
                sprite.collideStopYBottom( collision.overlapYBottom() );
                collide_top_ = true;
            }

            if ( collision.collideTop() )
            {
                collide_bottom_ = true;

                if ( !collide_top_prev_ )
                {
                    sprite.collideStopYTop( collision.overlapYTop() );
                }
            }
        }

        sprite.adjustBoundBoxes();
    };
